@@ -1,5 +1,5 @@
 /*                                                            -*- C -*-
- * Copyright (c) 2001-2004  Motoyuki Kasahara
+ * Copyright (c) 2001-2006  Motoyuki Kasahara
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -632,7 +632,7 @@ end_position={%d,%d})",
     context->zio = &book->subbook_current->sound_zio;
     context->location = start_location;
     if (start_location < end_location)
-	context->size = end_location - start_location;
+	context->size = end_location - start_location + 1;
     else {
 	error_code = EB_ERR_UNEXP_BINARY;
 	goto failed;
@@ -667,7 +667,14 @@ end_position={%d,%d})",
 
     if (memcmp(temporary_buffer, "fmt ", 4) == 0) {
 	memcpy(context->cache_buffer + 12, temporary_buffer, 4);
-	context->cache_length = 16;
+	if (zio_read(context->zio, context->cache_buffer + 16, 28) != 28) {
+	    error_code = EB_ERR_FAIL_READ_BINARY;
+	    goto failed;
+	}
+	if (context->size >= 32)
+	    context->size -= 32;
+	else
+	    context->size = 0;
     } else {
 	if (zio_lseek(context->zio,
 	    (book->subbook_current->sound.start_page - 1) * EB_SIZE_PAGE + 32,
@@ -689,7 +696,6 @@ end_position={%d,%d})",
 	*(unsigned char *)(context->cache_buffer + 43)
 	    = (context->size >> 24) & 0xff;
 
-	context->cache_length = 44;
 
 	/*
 	 * Seek sound file, again.
@@ -699,6 +705,7 @@ end_position={%d,%d})",
 	    goto failed;
 	}
     }
+    context->cache_length = 44;
 
     /*
      * Read and compose a WAVE header.
@@ -872,7 +879,7 @@ eb_set_binary_mpeg(EB_Book *book, const unsigned int *argv)
     LOG(("aux: eb_set_binary_mpeg(): movie_file_name=%s", movie_file_name));
 
     if (eb_find_file_name3(book->path, subbook->directory_name,
-	subbook->movie_directory_name, movie_file_name, movie_path_name)
+	subbook->movie_directory_name, movie_file_name, movie_file_name)
 	!= EB_SUCCESS) {
 	error_code = EB_ERR_NO_SUCH_BINARY;
 	goto failed;
@@ -1198,7 +1205,8 @@ eb_read_binary_mono_graphic(EB_Book *book, size_t binary_max_length,
 	 */
 	if (context->offset != 0
 	    && context->offset % line_length == 0
-	    && zio_lseek(context->zio, line_length * -2, SEEK_CUR) < 0) {
+	    && zio_lseek(context->zio, (off_t)line_length * -2, SEEK_CUR)
+	    < 0) {
 	    error_code = EB_ERR_FAIL_SEEK_BINARY;
 	    goto failed;
 	}
@@ -1328,7 +1336,8 @@ eb_read_binary_gray_graphic(EB_Book *book, size_t binary_max_length,
 	 */
 	if (context->offset != 0
 	    && context->offset % line_length == 0
-	    && zio_lseek(context->zio, line_length * -2, SEEK_CUR) < 0) {
+	    && zio_lseek(context->zio, (off_t)line_length * -2, SEEK_CUR)
+	    < 0) {
 		error_code = EB_ERR_FAIL_SEEK_BINARY;
 		goto failed;
 	}
